package com.persagy.dc.parser.service.impl;

import com.fasterxml.jackson.databind.JavaType;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.JsonNodeType;
import com.persagy.dc.basic.constant.DigitalDataCategory;
import com.persagy.dc.common.helper.SpringHelper;
import com.persagy.dc.common.model.data.RegionValue;
import com.persagy.dc.define.entity.ObjectInfo;
import com.persagy.dc.parser.entity.ParserResult;
import com.persagy.dc.parser.entity.RegionVO;
import com.persagy.dc.parser.service.IInfoDataParser;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * 信息点 抽象处理器
 * @author Charlie Yu
 * @date 2021-07-12
 */
public abstract class AbstractInfoDataParser implements IInfoDataParser {

    private static ObjectMapper objectMapper;

    /**
     * 接口实现
     * @param infoDefine 信息点 定义信息
     * @param valueInfo 信息点 值信息
     * @return
     */
    @Override
    public ParserResult parse(ObjectInfo infoDefine, JsonNode valueInfo) {
        ParserResult result = new ParserResult(infoDefine, valueInfo);
        // 校验信息点定义
        if(!validateDefine(result)) {
            return result;
        }
        // 空值、非静态信息点不需要值校验
        if(valueInfo == null ||
                JsonNodeType.NULL == valueInfo.getNodeType() ||
                DigitalDataCategory.STATIC.getIndex().equalsIgnoreCase(infoDefine.getCategory())) {
            return result;
        }
        // 值校验
        validateValue(result);
        return result;
    }

    /**
     * 信息点定义校验
     * @param result 校验参数
     * @return 是否校验通过
     */
    protected abstract boolean validateDefine(ParserResult result);

    /**
     * 信息点值校验
     * @param result 校验参数
     * @return 是否校验通过
     */
    protected abstract boolean validateValue(ParserResult result);

    /**
     * 获取数据
     * @param value
     * @param clazz
     * @param <T>
     * @return
     */
    protected <T> T formatValue(JsonNode value, Class<T> clazz) throws IOException {
        if(value == null || JsonNodeType.NULL == value.getNodeType()) {
            return null;
        }
        return objectMapper.readValue(value.toString(), clazz);
    }

    /**
     * 获取数据
     * @param value
     * @param clazz
     * @param <T>
     * @return
     */
    protected <T> List<T> formatListValue(JsonNode value, Class<T> clazz) throws IOException {
        if(value == null || JsonNodeType.NULL == value.getNodeType()) {
            return null;
        }
        JavaType javaType = objectMapper.getTypeFactory().constructParametricType(ArrayList.class, clazz);
        return objectMapper.readValue(value.toString(), javaType);
    }

    /**
     * 获取数据
     * @param value json
     * @param clazz 泛型类
     * @param <T>
     * @return RegionValue
     */
    protected <T extends Comparable> RegionValue<T> formatRegionValue(JsonNode value, Class<T> clazz) throws IOException {
        if(value == null || JsonNodeType.NULL == value.getNodeType()) {
            return null;
        }
        JavaType javaType = objectMapper.getTypeFactory().constructParametricType(RegionVO.class, clazz);
        RegionVO<T> region = objectMapper.readValue(value.toString(), javaType);
        return region == null ? null:region.toRegionValue();
    }

    /**
     * 获取数据
     * @param value
     * @param clazz 泛型类
     * @param <T>
     * @return List<RegionValue<T>>
     */
    protected <T extends Comparable> List<RegionValue<T>> formatRegionListValue(JsonNode value, Class<T> clazz) throws IOException {
        if(value == null || JsonNodeType.NULL == value.getNodeType()) {
            return null;
        }
        JavaType javaType = objectMapper.getTypeFactory().constructParametricType(RegionVO.class, clazz);
        javaType = objectMapper.getTypeFactory().constructParametricType(List.class, javaType);
        List<RegionVO<T>> regionList = objectMapper.readValue(value.toString(), javaType);
        return RegionVO.toRegionValueList(regionList);
    }

    /**
     * 获取ObjectMapper
     * @return
     */
    protected ObjectMapper getObjectMapper() {
        if(objectMapper == null) {
            objectMapper = SpringHelper.getBean(ObjectMapper.class);
        }
        return objectMapper;
    }
}
